==> dazuko-1.3.0-pre1.tar.gz: dazuko-1.3.0-pre1/dazuko_xp.h <==
/* DazukoXP. Allow cross platform file access control for 3rd-party applications.
   Written by John Ogness <jogness@antivir.de>

   Copyright (C) 2002-2003, H+BEDV Datentechnik GmbH
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

   1. Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   3. Neither the name of Dazuko nor the names of its contributors may be used
   to endorse or promote products derived from this software without specific
   prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef DAZUKO_XP_H
#define DAZUKO_XP_H

#define VERSION	"1.3.0-pre1"

#define	DAZUKO_FREE	0	/* the daemon is not ready */
#define	DAZUKO_READY	1	/* a daemon waits for something to do */
#define	DAZUKO_WAITING	2	/* a request is waiting to be served */
#define	DAZUKO_WORKING	3	/* daemon is currently in action */
#define	DAZUKO_DONE	4	/* daemon response is available */
#define	DAZUKO_BROKEN	5	/* invalid state (interrupt from ready,waiting) */


/*********************************************************
 * structures that MUST be implemented by platform-layer *
 *********************************************************/

struct xp_file;
struct xp_mutex;
struct xp_atomic;
struct xp_file_struct;
struct xp_queue;
struct xp_rwlock;


/******************************************
 * structures available to platform-layer *
 ******************************************/

struct slot_t
{
	/* A representation of a daemon. It holds
	 * all information about the daemon, the
	 * file that is scanned, and the state of
	 * the scanning process. */

	int			id;		
	int			pid;		/* pid of our daemon */
	int			state;
	int			response;
	int			event;
	int			o_flags;
	int			o_mode;
	int			kuid;		/* user id of the kernel process */
	int			kpid;		/* process id of the kernel process */
	int			filenamelength;	/* not including terminator */
	char			*filename;
	struct xp_mutex		mutex;
};

struct dazuko_file_struct
{
	/* A structure designed for simple and
	 * intelligent memory management when
	 * doing filename lookups in the kernel. */

	int			should_scan;		/* already know we need to scan? */
	char			*filename;		/* filename */
	int			filename_length;	/* length of filename */
	struct xp_file_struct	*extra_data;		/* extra platform-dependant data */
};


/********************************************************
 * functions that MUST be implemented by platform-layer *
 ********************************************************/

/* mutex */
inline int xp_init_mutex(struct xp_mutex *mutex);
inline int xp_down(struct xp_mutex *mutex);
inline int xp_up(struct xp_mutex *mutex);

/* read-write lock */
inline int xp_init_rwlock(struct xp_rwlock *rwlock);
inline int xp_write_lock(struct xp_rwlock *rwlock);
inline int xp_write_unlock(struct xp_rwlock *rwlock);
inline int xp_read_lock(struct xp_rwlock *rlock);
inline int xp_read_unlock(struct xp_rwlock *rlock);

/* wait-notify queue */
inline int xp_init_queue(struct xp_queue *queue);
inline int xp_wait_until_condition(struct xp_queue *queue, int (*cfunction)(void *), void *cparam);
inline int xp_notify(struct xp_queue *queue);

/* memory */
inline void* xp_malloc(int size);
inline int xp_free(void *ptr);

/* path attribute */
inline int xp_is_absolute_path(const char *path);

/* atomic */
inline int xp_atomic_set(struct xp_atomic *atomic, int value);
inline int xp_atomic_inc(struct xp_atomic *atomic);
inline int xp_atomic_dec(struct xp_atomic *atomic);
inline int xp_atomic_read(struct xp_atomic *atomic);

/* file descriptor */
inline int xp_copy_file(struct xp_file *dest, struct xp_file *src);
inline int xp_compare_file(struct xp_file *file1, struct xp_file *file2);

/* system hook */
inline int xp_sys_hook(void);
inline int xp_sys_unhook(void);

/* file structure */
inline int xp_file_struct_check(struct dazuko_file_struct *dfs);
inline int xp_file_struct_check_cleanup(struct dazuko_file_struct *dfs);

/* output */
inline int xp_print(const char *fmt, ...);

/* debug */
#ifdef DEBUG
#define DPRINT(fmt) xp_print fmt
#else
#define DPRINT(fmt)
#endif


/*****************************************
 * functions available to platform-layer *
 *****************************************/

int dazuko_unregister_daemon(int pid);
struct slot_t* dazuko_get_an_access(int pid);
int dazuko_return_access(int pid, int response);
int dazuko_set_option(int pid, int opt, void *param, int len);
int get_ready_slot_condition(void *param);
inline int dazuko_get_filename_length(char *filename);
inline int dazuko_state_error(struct slot_t *s, int current_state);
inline void dazuko_bzero(void *p, int len);
inline int dazuko_sys_exec_check(void);
inline int dazuko_sys_exec_pre(struct dazuko_file_struct *kfs, int o_flags, int o_mode, int pid, int uid);
inline int dazuko_sys_open_check(int pid);
inline int dazuko_sys_open_pre(struct dazuko_file_struct *kfs, int o_flags, int o_mode, int pid, int uid);
inline int dazuko_sys_open_post(struct dazuko_file_struct *kfs, struct xp_file *file, int o_flags, int o_mode);
inline int dazuko_sys_close_check(int pid);
inline int dazuko_sys_close_post(struct xp_file *file, int o_flags, int o_mode, int pid, int uid);
inline int dazuko_sys_write_check(void);
inline int dazuko_sys_write_post(struct xp_file *file);
inline int dazuko_init(void);
inline int dazuko_exit(void);

#endif

==> dazuko-2.0.0-pre4.tar.gz: dazuko-2.0.0-pre4/dazuko_xp.h <==
/* DazukoXP. Allow cross platform file access control for 3rd-party applications.
   Written by John Ogness <jogness@antivir.de>

   Copyright (c) 2002, 2003 H+BEDV Datentechnik GmbH
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

   1. Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   3. Neither the name of Dazuko nor the names of its contributors may be used
   to endorse or promote products derived from this software without specific
   prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef DAZUKO_XP_H
#define DAZUKO_XP_H

#define VERSION	"2.0.0-pre4"

#include "dazukoio_xp.h"

/* various requests */
#define SET_ACCESS_MASK		0
#define ADD_INCLUDE_PATH	1
#define ADD_EXCLUDE_PATH	2
#define REGISTER		3
#define REMOVE_ALL_PATHS	4
#define UNREGISTER		5
#define GET_AN_ACCESS		6
#define RETURN_AN_ACCESS	7

/* slot states */
#define	DAZUKO_FREE	0	/* the daemon is not ready */
#define	DAZUKO_READY	1	/* a daemon waits for something to do */
#define	DAZUKO_WAITING	2	/* a request is waiting to be served */
#define	DAZUKO_WORKING	3	/* daemon is currently in action */
#define	DAZUKO_DONE	4	/* daemon response is available */
#define	DAZUKO_BROKEN	5	/* invalid state (interrupt from ready,waiting) */


/*********************************************************
 * structures that MUST be implemented by platform-layer *
 *********************************************************/

/*
struct xp_file;
struct xp_mutex;
struct xp_atomic;
struct xp_file_struct;
struct xp_queue;
struct xp_rwlock;
*/


/******************************************
 * structures available to platform-layer *
 ******************************************/

struct daemon_id
{
	int	pid;
	int	unique;
};

struct event_properties
{
	int	flags;
	char	set_flags;
	int	mode;
	char	set_mode;
	int	uid;
	char	set_uid;
	int	pid;
	char	set_pid;
};

struct file_properties
{
	unsigned long	size;
	char		set_size;
	int		uid;
	char		set_uid;
	int		gid;
	char		set_gid;
	int		mode;
	char		set_mode;
	int		device_type;
	char		set_device_type;
};

struct slot
{
	/* A representation of a daemon. It holds
	 * all information about the daemon, the
	 * file that is scanned, and the state of
	 * the scanning process. */

	int			id;		
	struct daemon_id	did;		/* identifier for our daemon */
	int			write_mode;
	int			state;
	int			response;
	int			event;
	int			filenamelength;	/* not including terminator */
	char			*filename;
	struct event_properties	event_p;
	struct file_properties	file_p;
	struct xp_mutex		mutex;
};

struct dazuko_file_struct
{
	/* A structure designed for simple and
	 * intelligent memory management when
	 * doing filename lookups in the kernel. */

	int			should_scan;		/* already know we need to scan? */
	char			*filename;		/* filename */
	int			filename_length;	/* length of filename */
	struct file_properties	file_p;			/* properties of file */
	struct xp_file_struct	*extra_data;		/* extra platform-dependant data */
};


/********************************************************
 * functions that MUST be implemented by platform-layer *
 ********************************************************/

/* mutex */
inline int xp_init_mutex(struct xp_mutex *mutex);
inline int xp_down(struct xp_mutex *mutex);
inline int xp_up(struct xp_mutex *mutex);

/* read-write lock */
inline int xp_init_rwlock(struct xp_rwlock *rwlock);
inline int xp_write_lock(struct xp_rwlock *rwlock);
inline int xp_write_unlock(struct xp_rwlock *rwlock);
inline int xp_read_lock(struct xp_rwlock *rlock);
inline int xp_read_unlock(struct xp_rwlock *rlock);

/* wait-notify queue */
inline int xp_init_queue(struct xp_queue *queue);
inline int xp_wait_until_condition(struct xp_queue *queue, int (*cfunction)(void *), void *cparam, int allow_interrupt);
inline int xp_notify(struct xp_queue *queue);

/* memory */
inline void* xp_malloc(int size);
inline int xp_free(void *ptr);

/* path attribute */
inline int xp_is_absolute_path(const char *path);

/* atomic */
inline int xp_atomic_set(struct xp_atomic *atomic, int value);
inline int xp_atomic_inc(struct xp_atomic *atomic);
inline int xp_atomic_dec(struct xp_atomic *atomic);
inline int xp_atomic_read(struct xp_atomic *atomic);

/* file descriptor */
inline int xp_copy_file(struct xp_file *dest, struct xp_file *src);
inline int xp_compare_file(struct xp_file *file1, struct xp_file *file2);

/* system hook */
inline int xp_sys_hook(void);
inline int xp_sys_unhook(void);

/* file structure */
inline int xp_file_struct_check(struct dazuko_file_struct *dfs);
inline int xp_file_struct_check_cleanup(struct dazuko_file_struct *dfs);

/* output */
inline int xp_print(const char *fmt, ...);

/* debug */
#ifdef DEBUG
#define DPRINT(fmt) xp_print fmt
#else
#define DPRINT(fmt)
#endif


/*****************************************
 * functions available to platform-layer *
 *****************************************/

inline int dazuko_is_our_daemon(int pid);
int dazuko_get_value(const char *key, const char *string, char **value);
int dazuko_register_daemon(struct daemon_id *did, const char *reg_name, int string_length, int write_mode);
int dazuko_unregister_daemon(struct daemon_id *did);
struct slot* dazuko_get_an_access(struct daemon_id *did);
int dazuko_return_access(struct daemon_id *did, int response, struct slot *s);
int dazuko_handle_request(int pid, struct dazuko_request *request);
int dazuko_set_option(struct daemon_id *did, int opt, void *param, int len);
int get_ready_slot_condition(void *param);
inline int dazuko_get_filename_length(char *filename);
inline int dazuko_state_error(struct slot *s, int current_state);
inline void dazuko_bzero(void *p, int len);
inline int dazuko_sys_check(unsigned long event, int daemon_is_allowed, int pid);
inline int dazuko_sys_pre(unsigned long event, struct dazuko_file_struct *kfs, struct event_properties *event_p);
inline int dazuko_sys_post(unsigned long event, struct dazuko_file_struct *kfs, struct xp_file *file, struct event_properties *event_p);
inline int dazuko_init(void);
inline int dazuko_exit(void);

#endif
